# 物理内存管理和内核态虚拟内存管理
---
## 一、物理内存管理
在用户态程序中，内存的管理通常由操作系统负责，并且每个进程都会感知到一个独立的**虚拟地址空间**。
然而，在内核态，物理内存是**共享资源**，需要操作系统有效地管理。
因此，首先要确定物理内存的范围，通常是在内核代码加载后，剩余的内存部分。这个内存范围将被分割成多个**4KB**的物理页面，操作系统负责跟踪每个页面的**状态**（是否空闲、已分配等）。
## 二、虚拟内存管理
由于在内核中，物理内存并不直接映射到虚拟地址空间，尤其是内核代码、数据和用户进程的数据都需要有不同的虚拟地址空间映射。
为了处理虚拟地址的映射问题，引入了**页表**和**MMU（内存管理单元）**。
在最初的系统启动阶段，内存是没有分页的，因此所有地址访问都是直接访问物理地址。随着系统的进一步发展，**分页机制**被引入，允许操作系统通过页表来将虚拟地址转换为物理地址，这使得每个进程的虚拟地址空间得以隔离。
## 三、内核页表的设计
内核的虚拟地址空间通常是固定的，内核的代码、数据、内存分配区域等需要通过页表进行映射。
核心的内核页表结构包括：
- 内存映射寄存器：UART（串口）、CLINT（定时器和中断）、PLIC（中断控制）等硬件外设的映射，内核需要通过页表将虚拟地址映射到这些硬件寄存器的物理地址。
- 内核代码和数据空间：内核的代码段、数据段需要在内核页表中进行映射。
- 分配区域：操作系统需要为内核动态分配的内存区域（如堆、栈等）提供地址空间映射，确保内存使用的有效性和隔离性。
## 四、物理内存与虚拟内存的结合
在操作系统初始化时，需要通过一个全局的内核页表来管理内核的虚拟内存。
每个进程都有其自己的虚拟地址空间，而内核的虚拟地址空间在系统中是唯一的，通过页表来维护这些地址的转换关系。
操作系统需要确保所有的虚拟内存访问都经过适当的**映射**，确保系统稳定性和效率。
 ![alt text](image-1.png)