# 中断异常处理初步
![alt text](image-2.png)

---
## 一、RISC-V定义的Trap
RISC-V将Trap分为两类：
- Interrupt（中断）：外部事件触发，主要分为三类：
    - 时钟中断：由内置时钟硬件定期触发，用于周期性任务，如调度。
    - 软件中断：由其他核或软件发起，通常用于多核通信。
    - 外部中断：由外部设备（如UART、磁盘控制器）产生。
- Exception（异常）：由程序异常行为触发（如非法访问、算术错误等）。
    - 异常会导致程序暂停执行，进入内核进行处理。
## 二、三种权限模式的Trap处理
- M-mode（机器模式）：
    - RISC-V**最低层次**的权限模式，主要处理硬件相关的任务。
    - M-mode trap的主要功能是为S-mode设置环境（如时钟中断）。
    - 使用timerinit和timervec配合，更新时钟寄存器并产生一个S-mode软件中断。
- S-mode（监督模式）：
    - 处理自身的中断和异常。
    - 响应来自M-mode的S-mode软件中断。
    - 响应来自U-mode的委托中断和异常。
    - 核心处理函数：kerneltrap和usertrap。
- U-mode（用户模式）：
    - 用户模式本身不能直接处理trap。
    - 当用户模式发生中断或异常时，trap被委托给S-mode来处理，处理逻辑由usertrap负责。
## 三、时钟中断处理流程
时钟中断是内核支持的核心功能之一，用于周期性任务，如调度。处理流程如下：
- timerinit(in start.c)：
    - 设置时钟寄存器的初始状态；
    - 配置触发间隔（通常是100ms）。
- timervec(in kernelvec.S)：
    - 更新时钟寄存器状态，准备下一次中断；
    - 触发S-mode trap。
- kerneltrap(in trap.c)：
    - 检查中断源是否为时钟中断；
    - 更新ticks（时钟计数器）；
    - 唤醒等待时钟的进程。
## 四、外设中断处理流程
外设中断（如UAR或磁盘）通过PLIC（Platform-Level Interrupt Controller）管理。处理流程：
- plicinit和plicinithart(in plic.c)：
    - 设置外设的中断优先级；
    - 启用硬件线程（hart）的中断处理。
- kerneltrap(in trap.c)：
    - 判断中断是否来自外设（通过devintr）；
    - 调用具体设备的中断处理函数。
- 设备中断处理函数：uartintr(in uart.c)：
    - 处理UART的输入输出；virtio_disk_intr(in virtio_disk.c)：
    - 处理磁盘的读写请求。
