# 文件系统的底层设备和中层抽象
---
## 一、文件系统的操作流程
- 首先在fsinit()中初始化文件系统，读取超级块验证其有效性并初始化日志模块。
- 磁盘块通过balloc()分配并清空，bfree()释放。
- 文件由inode管理，使用ialloc()分配、iupdate()更新，readi()和writei()执行文件数据操作。
- 目录项通过特殊目录inode管理，dirlookup()查找，dirlink()添加新项。- 路径解析通过skipelem()逐层查找路径元素对应的inode。
- 日志系统使用log_write()记录磁盘操作，确保数据一致性和崩溃恢复。
- 之外，操作中包括异常处理，防止非法操作，如重复释放磁盘块或空inode等。
 
![alt text](image-10.png)

## 二、磁盘布局与磁盘驱动
- 1.磁盘映像文件的创建依赖外部的linux环境，因此mkfs.c应当独立放在一个目录下。
    - 简单来说，磁盘映像文件的创建过程即是使用linux提供的open+read+write+lseek系统调用生成一个文件。
- 2.在xv6的文件系统中，磁盘被组织为多个块（block）。文件系统会使用磁盘块来存储文件数据、目录项、以及文件系统的元数据（如超级块和位图）。
磁盘管理在文件系统中有两个主要任务：
    - 块分配：通过位图（bitmap）管理磁盘的使用情况，决定哪些块是空闲的，哪些块已经被使用。
    - 超级块（Superblock）：记录文件系统的元数据，如文件系统的大小、磁盘块的分配情况、空闲块的信息等。

## 三、inode在内存和磁盘中的区别与联系
inode是用于存储文件元数据（metadata）的数据结构。它包含了文件的基本信息，如文件的大小、创建时间、修改时间、文件类型、权限、硬盘块的指针等。
在内存和磁盘中，它门的设计与实现是需要区分的。

![alt text](image-11.png)
 
## 四、目录项管理
- 目录项用于管理文件系统中的文件和子目录。每个目录项由一个**结构体dirent**表示，包含文件名和相应的inode编号。
- 目录本身是一个特殊的文件，其内容是一个包含若干目录项的数组，每个目录项对应一个文件或子目录。
- 当对目录进行读取时，内核会遍历该数组，解析其中的目录项并返回给用户。目录项的管理通过inode的映射关系进行，
- 目录的操作（如添加、删除文件）会更新相应inode的指针与状态信息。
    - 每个目录项存放的信息：
    - 该目录对应的inode序号
    - 目录名    
- 四种基本操作：
    - add_entry
    - delete_entry
    - search_entry
    - get_entries

## 五、文件路径解析

文件路径解析在xv6中至关重要，它决定了系统如何通过用户提供的路径准确定位到**文件**或**目录**。
路径解析涉及从**字符串形式**的路径转换为文件系统中的实际存储位置，确保操作系统能够正确地识别和访问文件。
在解析过程中，操作系统需要遍历路径中的每个目录项，逐层查找并访问文件系统结构，最终返回对应的inode。
这一过程确保了文件系统的层次结构得以正确管理，并为文件的读写、创建、删除等操作提供了基础。
因此，路径解析不仅是文件系统实现的核心环节，也是操作系统管理存储、处理用户请求的关键机制。
 
![alt text](image-12.png)
