#include "kernel/types.h"
#include "kernel/stat.h"
#include "user/user.h"

volatile int lock = 0; // 全局锁

void acquire_lock() {
    while (__sync_lock_test_and_set(&lock, 1)) {
        // busy-wait
    }
}

void release_lock() {
    __sync_lock_release(&lock);
}

// 子进程任务
void child_task(int priority) {
    for (int i = 0; i < 3; i++) {
        sleep(20);  // 模拟运行
    }
    // 打印任务完成信息，加锁确保输出完整
    acquire_lock();
    printf("priority=%d finished.\n", priority);
    release_lock();
    exit(0);
}

int main() {
    int priorities[] = {30, 50,10}; // 手动设置优先级
    int num_processes = 3; // 创建3个子进程

    printf("Priority Scheduling Test: Starting...\n");

    for (int i = 0; i < num_processes; i++) {
        int pid = fork();
        if (pid == 0) {
            // 子进程设置优先级并运行任务
            setpriority(priorities[i]);
            child_task(priorities[i]);
        }
    }

    for (int i = 0; i < num_processes; i++) {
        wait(0); // 等待所有子进程完成
    }

    printf("Priority Scheduling Test: Finished.\n");
    exit(0);
}










