#include "kernel/types.h"
#include "user/user.h"

#define NUM_THREADS 10

// 用户态自定义的 spinlock
struct spinlock {
  uint locked; // 0: unlocked, 1: locked
  char *name;  // Lock name
};

// 初始化锁
void initlock(struct spinlock *lk, char *name) {
  lk->locked = 0;
  lk->name = name;
}

// 获取锁
void acquire(struct spinlock *lk) {
  while (__sync_lock_test_and_set(&lk->locked, 1)) {
    ; // 自旋等待锁释放
  }
}

// 释放锁
void release(struct spinlock *lk) {
  __sync_lock_release(&lk->locked);
}

struct spinlock lock;
volatile int counter = 0;

void thread_func(void *arg) {
  for (int i = 0; i < 1000; i++) {
    acquire(&lock);  // 获取锁
    counter++;       // 临界区
    release(&lock);  // 释放锁
  }
  exit(0);
}

int main() {
  initlock(&lock, "test_lock"); // 初始化锁

  for (int i = 0; i < NUM_THREADS; i++) {
    if (fork() == 0) { // 创建多个线程
      thread_func(0);
    }
  }

  for (int i = 0; i < NUM_THREADS; i++) {
    wait(0); // 等待子进程结束
  }

  // 验证最终计数器的值
  printf("Final counter value: %d (expected: %d)\n", counter, NUM_THREADS * 1000);

  exit(0);
}
